@@ -23,21 +23,25 @@
 package org.jboss.as.clustering.infinispan.subsystem;
 
 import static org.jboss.as.clustering.infinispan.subsystem.CacheContainerResourceDefinition.Attribute.DEFAULT_CACHE;
+import static org.jboss.as.clustering.infinispan.subsystem.CacheContainerResourceDefinition.Attribute.MARSHALLER;
 import static org.jboss.as.clustering.infinispan.subsystem.CacheContainerResourceDefinition.Attribute.STATISTICS_ENABLED;
 import static org.jboss.as.clustering.infinispan.subsystem.CacheContainerResourceDefinition.Capability.CONFIGURATION;
 
-import java.util.ArrayList;
+import java.io.File;
+import java.io.UncheckedIOException;
 import java.util.EnumMap;
 import java.util.EnumSet;
 import java.util.List;
 import java.util.Map;
-import java.util.NoSuchElementException;
+import java.util.ServiceLoader;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Supplier;
+import java.util.stream.Collectors;
 
 import javax.management.MBeanServer;
 
+import org.infinispan.commands.module.ModuleCommandExtensions;
 import org.infinispan.commons.marshall.Marshaller;
 import org.infinispan.commons.util.AggregatedClassLoader;
 import org.infinispan.configuration.global.GlobalConfiguration;
@@ -46,26 +50,27 @@ import org.infinispan.configuration.global.ThreadPoolConfiguration;
 import org.infinispan.configuration.global.TransportConfiguration;
 import org.infinispan.configuration.internal.PrivateGlobalConfigurationBuilder;
 import org.infinispan.globalstate.ConfigurationStorage;
+import org.infinispan.protostream.SerializationContext;
+import org.infinispan.protostream.SerializationContextInitializer;
 import org.jboss.as.clustering.controller.CapabilityServiceNameProvider;
 import org.jboss.as.clustering.controller.CommonRequirement;
 import org.jboss.as.clustering.controller.ResourceServiceConfigurator;
-import org.jboss.as.clustering.infinispan.InfinispanLogger;
-import org.jboss.as.clustering.infinispan.MBeanServerProvider;
+import org.jboss.as.clustering.infinispan.jmx.MBeanServerProvider;
+import org.jboss.as.clustering.infinispan.logging.InfinispanLogger;
 import org.jboss.as.controller.OperationContext;
 import org.jboss.as.controller.OperationFailedException;
 import org.jboss.as.controller.PathAddress;
+import org.jboss.as.server.ServerEnvironment;
+import org.jboss.as.server.ServerEnvironmentService;
 import org.jboss.as.server.Services;
 import org.jboss.dmr.ModelNode;
-import org.jboss.marshalling.ModularClassResolver;
 import org.jboss.modules.Module;
 import org.jboss.modules.ModuleLoader;
 import org.jboss.msc.Service;
 import org.jboss.msc.service.ServiceBuilder;
 import org.jboss.msc.service.ServiceController;
 import org.jboss.msc.service.ServiceTarget;
-import org.wildfly.clustering.infinispan.marshalling.jboss.JBossMarshaller;
-import org.wildfly.clustering.infinispan.spi.marshalling.InfinispanProtoStreamMarshaller;
-import org.wildfly.clustering.marshalling.protostream.ModuleClassLoaderMarshaller;
+import org.wildfly.clustering.infinispan.marshall.InfinispanMarshallerFactory;
 import org.wildfly.clustering.service.CompositeDependency;
 import org.wildfly.clustering.service.Dependency;
 import org.wildfly.clustering.service.FunctionalService;
@@ -81,6 +86,7 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
     private final SupplierDependency<ModuleLoader> loader;
     private final SupplierDependency<List<Module>> modules;
     private final SupplierDependency<TransportConfiguration> transport;
+    private final SupplierDependency<ServerEnvironment> environment;
     private final Map<ThreadPoolResourceDefinition, SupplierDependency<ThreadPoolConfiguration>> pools = new EnumMap<>(ThreadPoolResourceDefinition.class);
     private final Map<ScheduledThreadPoolResourceDefinition, SupplierDependency<ThreadPoolConfiguration>> scheduledPools = new EnumMap<>(ScheduledThreadPoolResourceDefinition.class);
     private final String name;
@@ -88,6 +94,7 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
     private volatile SupplierDependency<MBeanServer> server;
     private volatile String defaultCache;
     private volatile boolean statisticsEnabled;
+    private volatile InfinispanMarshallerFactory marshallerFactory;
 
     GlobalConfigurationServiceConfigurator(PathAddress address) {
         super(CONFIGURATION, address);
@@ -95,6 +102,7 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
         this.loader = new ServiceSupplierDependency<>(Services.JBOSS_SERVICE_MODULE_LOADER);
         this.modules = new ServiceSupplierDependency<>(CacheContainerComponent.MODULES.getServiceName(address));
         this.transport = new ServiceSupplierDependency<>(CacheContainerComponent.TRANSPORT.getServiceName(address));
+        this.environment = new ServiceSupplierDependency<>(ServerEnvironmentService.SERVICE_NAME);
         for (ThreadPoolResourceDefinition pool : EnumSet.of(ThreadPoolResourceDefinition.LISTENER, ThreadPoolResourceDefinition.BLOCKING, ThreadPoolResourceDefinition.NON_BLOCKING)) {
             this.pools.put(pool, new ServiceSupplierDependency<>(pool.getServiceName(address)));
         }
@@ -108,6 +116,7 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
         this.server = context.hasOptionalCapability(CommonRequirement.MBEAN_SERVER.getName(), null, null) ? new ServiceSupplierDependency<>(CommonRequirement.MBEAN_SERVER.getServiceName(context)) : null;
         this.defaultCache = DEFAULT_CACHE.resolveModelAttribute(context, model).asStringOrNull();
         this.statisticsEnabled = STATISTICS_ENABLED.resolveModelAttribute(context, model).asBoolean();
+        this.marshallerFactory = InfinispanMarshallerFactory.valueOf(MARSHALLER.resolveModelAttribute(context, model).asString());
         return this;
     }
 
@@ -122,14 +131,33 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
         builder.transport().read(this.transport.get());
 
         List<Module> modules = this.modules.get();
-        List<ClassLoader> loaders = new ArrayList<>(modules.size());
-        for (Module module : modules) {
-            loaders.add(module.getClassLoader());
-        }
-        ClassLoader loader = loaders.size() > 1 ? new AggregatedClassLoader(loaders) : loaders.get(0);
-        Marshaller marshaller = this.createMarshaller(loader);
+        Marshaller marshaller = this.marshallerFactory.apply(this.loader.get(), modules);
         InfinispanLogger.ROOT_LOGGER.debugf("%s cache-container will use %s", this.name, marshaller.getClass().getName());
-        builder.serialization().marshaller(marshaller);
+        // Register dummy serialization context initializer, to bypass service loading in org.infinispan.marshall.protostream.impl.SerializationContextRegistryImpl
+        // Otherwise marshaller auto-detection will not work
+        builder.serialization().marshaller(marshaller).addContextInitializer(new SerializationContextInitializer() {
+            @Deprecated
+            @Override
+            public String getProtoFile() throws UncheckedIOException {
+                return null;
+            }
+
+            @Deprecated
+            @Override
+            public String getProtoFileName() {
+                return null;
+            }
+
+            @Override
+            public void registerMarshallers(SerializationContext context) {
+            }
+
+            @Override
+            public void registerSchema(SerializationContext context) {
+            }
+        });
+
+        ClassLoader loader = modules.size() > 1 ? new AggregatedClassLoader(modules.stream().map(Module::getClassLoader).collect(Collectors.toList())) : modules.get(0).getClassLoader();
         builder.classLoader(loader);
 
         builder.blockingThreadPool().read(this.pools.get(ThreadPoolResourceDefinition.BLOCKING).get());
@@ -139,30 +167,33 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
 
         builder.shutdown().hookBehavior(ShutdownHookBehavior.DONT_REGISTER);
         // Disable registration of MicroProfile Metrics
-        builder.metrics().gauges(false).histograms(false);
+        builder.metrics().gauges(false).histograms(false).accurateSize(true);
         builder.jmx().domain("org.wildfly.clustering.infinispan")
                 .mBeanServerLookup((this.server != null) ? new MBeanServerProvider(this.server.get()) : null)
                 .enabled(this.server != null)
                 ;
 
+        // Disable triangle algorithm - we optimize for originator as primary owner
         // Do not enable server-mode for the Hibernate 2LC use case:
         // * The 2LC stack already overrides the interceptor for distribution caches
         // * This renders Infinispan default 2LC configuration unusable as it results in a default media type of application/unknown for keys and values
         // See ISPN-12252 for details
-        if (modules.stream().map(Module::getName).noneMatch("org.infinispan.hibernate-cache"::equals)) {
-            // Disable triangle algorithm - we optimize for originator as primary owner
-            builder.addModule(PrivateGlobalConfigurationBuilder.class).serverMode(true);
-        }
-        // Disable configuration storage
-        builder.globalState().configurationStorage(ConfigurationStorage.IMMUTABLE).disable();
+        builder.addModule(PrivateGlobalConfigurationBuilder.class).serverMode(!ServiceLoader.load(ModuleCommandExtensions.class, loader).iterator().hasNext());
 
+        String path = InfinispanExtension.SUBSYSTEM_NAME + File.separatorChar + this.name;
+        ServerEnvironment environment = this.environment.get();
+        builder.globalState().enable()
+                .configurationStorage(ConfigurationStorage.VOLATILE)
+                .persistentLocation(path, environment.getServerDataDir().getPath())
+                .temporaryLocation(path, environment.getServerTempDir().getPath())
+                ;
         return builder.build();
     }
 
     @Override
     public ServiceBuilder<?> build(ServiceTarget target) {
         ServiceBuilder<?> builder = target.addService(this.getServiceName());
-        Consumer<GlobalConfiguration> global = new CompositeDependency(this.loader, this.modules, this.transport, this.server).register(builder).provides(this.getServiceName());
+        Consumer<GlobalConfiguration> global = new CompositeDependency(this.loader, this.modules, this.transport, this.server, this.environment).register(builder).provides(this.getServiceName());
         for (Dependency dependency: this.pools.values()) {
             dependency.register(builder);
         }
@@ -172,13 +203,4 @@ public class GlobalConfigurationServiceConfigurator extends CapabilityServiceNam
         Service service = new FunctionalService<>(global, Function.identity(), this);
         return builder.setInstance(service).setInitialMode(ServiceController.Mode.PASSIVE);
     }
-
-    private Marshaller createMarshaller(ClassLoader loader) {
-        ModuleLoader moduleLoader = this.loader.get();
-        try {
-            return new InfinispanProtoStreamMarshaller(new ModuleClassLoaderMarshaller(moduleLoader), loader);
-        } catch (NoSuchElementException e) {
-            return new JBossMarshaller(ModularClassResolver.getInstance(moduleLoader), loader);
-        }
-    }
 }
