@@ -24,66 +24,36 @@ package org.jboss.as.clustering.infinispan.subsystem;
 
 import static org.jboss.as.clustering.infinispan.subsystem.CacheContainerResourceDefinition.Capability.KEY_AFFINITY_FACTORY;
 
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
 import java.util.function.Consumer;
-import java.util.function.Function;
-import java.util.function.Supplier;
 
-import org.infinispan.affinity.KeyAffinityService;
-import org.jboss.as.clustering.context.DefaultExecutorService;
-import org.jboss.as.clustering.context.DefaultThreadFactory;
 import org.jboss.as.clustering.controller.CapabilityServiceNameProvider;
 import org.jboss.as.controller.PathAddress;
 import org.jboss.msc.Service;
 import org.jboss.msc.service.ServiceBuilder;
 import org.jboss.msc.service.ServiceController;
+import org.jboss.msc.service.ServiceName;
 import org.jboss.msc.service.ServiceTarget;
-import org.wildfly.clustering.infinispan.spi.affinity.DefaultKeyAffinityServiceFactory;
-import org.wildfly.clustering.infinispan.spi.affinity.KeyAffinityServiceFactory;
-import org.wildfly.clustering.service.AsyncServiceConfigurator;
-import org.wildfly.clustering.service.FunctionalService;
+import org.wildfly.clustering.infinispan.affinity.KeyAffinityServiceFactory;
+import org.wildfly.clustering.infinispan.affinity.impl.DefaultKeyAffinityServiceFactory;
 import org.wildfly.clustering.service.ServiceConfigurator;
-import org.wildfly.security.manager.WildFlySecurityManager;
 
 /**
  * Key affinity service factory that will only generates keys for use by the local node.
  * Returns a trivial implementation if the specified cache is not distributed.
  * @author Paul Ferraro
  */
-public class KeyAffinityServiceFactoryServiceConfigurator extends CapabilityServiceNameProvider implements ServiceConfigurator, Function<ExecutorService, KeyAffinityServiceFactory>, Supplier<ExecutorService>, Consumer<ExecutorService> {
-
-    private volatile int bufferSize = 100;
+public class KeyAffinityServiceFactoryServiceConfigurator extends CapabilityServiceNameProvider implements ServiceConfigurator {
 
     public KeyAffinityServiceFactoryServiceConfigurator(PathAddress address) {
         super(KEY_AFFINITY_FACTORY, address);
     }
 
-    public KeyAffinityServiceFactoryServiceConfigurator setBufferSize(int size) {
-        this.bufferSize = size;
-        return this;
-    }
-
-    @Override
-    public ExecutorService get() {
-        return Executors.newCachedThreadPool(new DefaultThreadFactory(KeyAffinityService.class));
-    }
-
-    @Override
-    public KeyAffinityServiceFactory apply(ExecutorService executor) {
-        return new DefaultKeyAffinityServiceFactory(executor, this.bufferSize);
-    }
-
-    @Override
-    public void accept(ExecutorService executor) {
-        WildFlySecurityManager.doUnchecked(executor, DefaultExecutorService.SHUTDOWN_NOW_ACTION);
-    }
-
     @Override
     public ServiceBuilder<?> build(ServiceTarget target) {
-        ServiceBuilder<?> builder = new AsyncServiceConfigurator(this.getServiceName()).startSynchronously().build(target);
-        Consumer<KeyAffinityServiceFactory> affinityFactory = builder.provides(this.getServiceName());
-        Service service = new FunctionalService<>(affinityFactory, this, this, this);
+        ServiceName name = this.getServiceName();
+        ServiceBuilder<?> builder = target.addService(name);
+        Consumer<KeyAffinityServiceFactory> affinityFactory = builder.provides(name);
+        Service service = Service.newInstance(affinityFactory, new DefaultKeyAffinityServiceFactory());
         return builder.setInstance(service).setInitialMode(ServiceController.Mode.ON_DEMAND);
     }
 }
