@@ -25,8 +25,6 @@ package org.wildfly.clustering.web.cache.session;
 import java.util.AbstractMap.SimpleImmutableEntry;
 import java.util.Map;
 
-import javax.servlet.ServletContext;
-
 import org.wildfly.clustering.web.LocalContextFactory;
 import org.wildfly.clustering.web.session.ImmutableSession;
 import org.wildfly.clustering.web.session.ImmutableSessionAttributes;
@@ -34,15 +32,18 @@ import org.wildfly.clustering.web.session.ImmutableSessionMetaData;
 import org.wildfly.clustering.web.session.Session;
 
 /**
+ * @param <C> the ServletContext specification type
+ * @param <V> the session attribute value type
+ * @param <L> the local context type
  * @author Paul Ferraro
  */
-public class CompositeSessionFactory<V, L> extends CompositeImmutableSessionFactory<V, L> implements SessionFactory<CompositeSessionMetaDataEntry<L>, V, L> {
+public class CompositeSessionFactory<C, V, L> extends CompositeImmutableSessionFactory<V, L> implements SessionFactory<C, CompositeSessionMetaDataEntry<L>, V, L> {
 
     private final SessionMetaDataFactory<CompositeSessionMetaDataEntry<L>> metaDataFactory;
-    private final SessionAttributesFactory<V> attributesFactory;
+    private final SessionAttributesFactory<C, V> attributesFactory;
     private final LocalContextFactory<L> localContextFactory;
 
-    public CompositeSessionFactory(SessionMetaDataFactory<CompositeSessionMetaDataEntry<L>> metaDataFactory, SessionAttributesFactory<V> attributesFactory, LocalContextFactory<L> localContextFactory) {
+    public CompositeSessionFactory(SessionMetaDataFactory<CompositeSessionMetaDataEntry<L>> metaDataFactory, SessionAttributesFactory<C, V> attributesFactory, LocalContextFactory<L> localContextFactory) {
         super(metaDataFactory, attributesFactory);
         this.metaDataFactory = metaDataFactory;
         this.attributesFactory = attributesFactory;
@@ -50,10 +51,10 @@ public class CompositeSessionFactory<V, L> extends CompositeImmutableSessionFact
     }
 
     @Override
-    public Map.Entry<CompositeSessionMetaDataEntry<L>, V> createValue(String id, Void context) {
-        CompositeSessionMetaDataEntry<L> metaDataValue = this.metaDataFactory.createValue(id, context);
+    public Map.Entry<CompositeSessionMetaDataEntry<L>, V> createValue(String id, SessionCreationMetaData creationMetaData) {
+        CompositeSessionMetaDataEntry<L> metaDataValue = this.metaDataFactory.createValue(id, creationMetaData);
         if (metaDataValue == null) return null;
-        V attributesValue = this.attributesFactory.createValue(id, context);
+        V attributesValue = this.attributesFactory.createValue(id, null);
         return new SimpleImmutableEntry<>(metaDataValue, attributesValue);
     }
 
@@ -85,20 +86,14 @@ public class CompositeSessionFactory<V, L> extends CompositeImmutableSessionFact
 
     @Override
     public boolean remove(String id) {
-        if (this.metaDataFactory.remove(id)) {
-            this.attributesFactory.remove(id);
-            return true;
-        }
-        return false;
+        this.attributesFactory.remove(id);
+        return this.metaDataFactory.remove(id);
     }
 
     @Override
     public boolean purge(String id) {
-        if (this.metaDataFactory.purge(id)) {
-            this.attributesFactory.purge(id);
-            return true;
-        }
-        return false;
+        this.attributesFactory.purge(id);
+        return this.metaDataFactory.purge(id);
     }
 
     @Override
@@ -107,12 +102,12 @@ public class CompositeSessionFactory<V, L> extends CompositeImmutableSessionFact
     }
 
     @Override
-    public SessionAttributesFactory<V> getAttributesFactory() {
+    public SessionAttributesFactory<C, V> getAttributesFactory() {
         return this.attributesFactory;
     }
 
     @Override
-    public Session<L> createSession(String id, Map.Entry<CompositeSessionMetaDataEntry<L>, V> entry, ServletContext context) {
+    public Session<L> createSession(String id, Map.Entry<CompositeSessionMetaDataEntry<L>, V> entry, C context) {
         CompositeSessionMetaDataEntry<L> key = entry.getKey();
         InvalidatableSessionMetaData metaData = this.metaDataFactory.createSessionMetaData(id, key);
         SessionAttributes attributes = this.attributesFactory.createSessionAttributes(id, entry.getValue(), metaData, context);
@@ -123,4 +118,10 @@ public class CompositeSessionFactory<V, L> extends CompositeImmutableSessionFact
     public ImmutableSession createImmutableSession(String id, ImmutableSessionMetaData metaData, ImmutableSessionAttributes attributes) {
         return new CompositeImmutableSession(id, metaData, attributes);
     }
+
+    @Override
+    public void close() {
+        this.metaDataFactory.close();
+        this.attributesFactory.close();
+    }
 }
