@@ -23,6 +23,8 @@ package org.jboss.as.connector.subsystems.resourceadapters;
 
 import static javax.xml.stream.XMLStreamConstants.END_ELEMENT;
 import static javax.xml.stream.XMLStreamConstants.START_ELEMENT;
+import static org.jboss.as.connector.subsystems.common.jndi.Constants.JNDI_NAME;
+import static org.jboss.as.connector.subsystems.common.jndi.Constants.USE_JAVA_CONTEXT;
 import static org.jboss.as.connector.subsystems.common.pool.Constants.BACKGROUNDVALIDATION;
 import static org.jboss.as.connector.subsystems.common.pool.Constants.BACKGROUNDVALIDATIONMILLIS;
 import static org.jboss.as.connector.subsystems.common.pool.Constants.BLOCKING_TIMEOUT_WAIT_MILLIS;
@@ -53,14 +55,13 @@ import static org.jboss.as.connector.subsystems.resourceadapters.Constants.ENABL
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.ENLISTMENT;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.ENLISTMENT_TRACE;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.INTERLEAVING;
-import static org.jboss.as.connector.subsystems.resourceadapters.Constants.JNDINAME;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.MCP;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.NOTXSEPARATEPOOL;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.NO_RECOVERY;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.PAD_XID;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.POOL_NAME_NAME;
-import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVERLUGIN_CLASSNAME;
-import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVERLUGIN_PROPERTIES;
+import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVER_PLUGIN_CLASSNAME;
+import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVER_PLUGIN_PROPERTIES;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVERY_AUTHENTICATION_CONTEXT;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVERY_CREDENTIAL_REFERENCE;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.RECOVERY_ELYTRON_ENABLED;
@@ -73,7 +74,6 @@ import static org.jboss.as.connector.subsystems.resourceadapters.Constants.SECUR
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.SHARABLE;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.TRACKING;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.USE_CCM;
-import static org.jboss.as.connector.subsystems.resourceadapters.Constants.USE_JAVA_CONTEXT;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.WRAP_XA_RESOURCE;
 import static org.jboss.as.connector.subsystems.resourceadapters.Constants.XA_RESOURCE_TIMEOUT;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.ADD;
@@ -137,10 +137,10 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
      * parse a single connection-definition tag
      *
      * @param reader the reader
-     * @throws javax.xml.stream.XMLStreamException
+     * @throws XMLStreamException
      *                         XMLStreamException
      * @throws ParserException ParserException
-     * @throws org.jboss.jca.common.api.validator.ValidateException
+     * @throws ValidateException
      *                         ValidateException
      */
     protected void parseConnectionDefinitions_3_0(final XMLExtendedStreamReader reader, final Map<String, ModelNode> map,
@@ -177,7 +177,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                 }
                 case JNDI_NAME: {
                     jndiName = value;
-                    JNDINAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
+                    JNDI_NAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
                     break;
                 }
                 case POOL_NAME: {
@@ -254,7 +254,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                             break;
                         }
                         case TIMEOUT: {
-                            parseTimeOut(reader, isXa, connectionDefinitionNode);
+                            parseTimeOut(reader, connectionDefinitionNode);
                             break;
                         }
                         case VALIDATION: {
@@ -302,10 +302,10 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
      * parse a single connection-definition tag
      *
      * @param reader the reader
-     * @throws javax.xml.stream.XMLStreamException
+     * @throws XMLStreamException
      *                         XMLStreamException
      * @throws ParserException ParserException
-     * @throws org.jboss.jca.common.api.validator.ValidateException
+     * @throws ValidateException
      *                         ValidateException
      */
     protected void parseConnectionDefinitions_4_0(final XMLExtendedStreamReader reader, final Map<String, ModelNode> map,
@@ -342,7 +342,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                 }
                 case JNDI_NAME: {
                     jndiName = value;
-                    JNDINAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
+                    JNDI_NAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
                     break;
                 }
                 case POOL_NAME: {
@@ -426,7 +426,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                             break;
                         }
                         case TIMEOUT: {
-                            parseTimeOut(reader, isXa, connectionDefinitionNode);
+                            parseTimeOut(reader, connectionDefinitionNode);
                             break;
                         }
                         case VALIDATION: {
@@ -504,7 +504,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                 }
                 case JNDI_NAME: {
                     jndiName = value;
-                    JNDINAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
+                    JNDI_NAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
                     break;
                 }
                 case POOL_NAME: {
@@ -565,10 +565,10 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
      * parse a single connection-definition tag
      *
      * @param reader the reader
-     * @throws javax.xml.stream.XMLStreamException
+     * @throws XMLStreamException
      *                         XMLStreamException
      * @throws ParserException ParserException
-     * @throws org.jboss.jca.common.api.validator.ValidateException
+     * @throws ValidateException
      *                         ValidateException
      */
     protected void parseConnectionDefinitions_5_0(final XMLExtendedStreamReader reader, final Map<String, ModelNode> map,
@@ -610,7 +610,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                             break;
                         }
                         case TIMEOUT: {
-                            parseTimeOut(reader, isXa, connectionDefinitionNode);
+                            parseTimeOut(reader, connectionDefinitionNode);
                             break;
                         }
                         case VALIDATION: {
@@ -658,10 +658,10 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
          * parse a single connection-definition tag
          *
          * @param reader the reader
-         * @throws javax.xml.stream.XMLStreamException
+         * @throws XMLStreamException
          *                         XMLStreamException
          * @throws ParserException ParserException
-         * @throws org.jboss.jca.common.api.validator.ValidateException
+         * @throws ValidateException
          *                         ValidateException
          */
         protected void parseConnectionDefinitions_1_0(final XMLExtendedStreamReader reader, final Map<String, ModelNode> map,
@@ -688,7 +688,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                     }
                     case JNDI_NAME: {
                         jndiName = value;
-                        JNDINAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
+                        JNDI_NAME.parseAndSetParameter(jndiName, connectionDefinitionNode, reader);
                         break;
                     }
                     case POOL_NAME: {
@@ -765,7 +765,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                                 break;
                             }
                             case TIMEOUT: {
-                                parseTimeOut(reader, isXa, connectionDefinitionNode);
+                                parseTimeOut(reader, connectionDefinitionNode);
                                 break;
                             }
                             case VALIDATION: {
@@ -859,7 +859,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
         throw ParseUtils.unexpectedEndElement(reader);
     }
 
-    private void parseTimeOut(XMLExtendedStreamReader reader, Boolean isXa, ModelNode node) throws XMLStreamException,
+    private void parseTimeOut(XMLExtendedStreamReader reader, ModelNode node) throws XMLStreamException,
             ParserException, ValidateException {
 
         while (reader.hasNext()) {
@@ -932,9 +932,9 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                     break;
                 }
                 case JNDI_NAME: {
-                    jndiName = rawAttributeText(reader, JNDINAME.getXmlName());
+                    jndiName = rawAttributeText(reader, JNDI_NAME.getXmlName());
                     if (jndiName != null) {
-                        JNDINAME.parseAndSetParameter(jndiName, adminObjectNode, reader);
+                        JNDI_NAME.parseAndSetParameter(jndiName, adminObjectNode, reader);
                     }
                     break;
                 }
@@ -1255,7 +1255,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                             break;
                         }
                         case RECOVER_PLUGIN: {
-                            parseExtension(reader, tag.getLocalName(), node, RECOVERLUGIN_CLASSNAME, RECOVERLUGIN_PROPERTIES);
+                            parseExtension(reader, tag.getLocalName(), node, RECOVER_PLUGIN_CLASSNAME, RECOVER_PLUGIN_PROPERTIES);
                             break;
                         }
                         default:
@@ -1305,7 +1305,7 @@ public abstract class CommonIronJacamarParser extends AbstractParser {
                             break;
                         }
                         case RECOVER_PLUGIN: {
-                            parseExtension(reader, tag.getLocalName(), node, RECOVERLUGIN_CLASSNAME, RECOVERLUGIN_PROPERTIES);
+                            parseExtension(reader, tag.getLocalName(), node, RECOVER_PLUGIN_CLASSNAME, RECOVER_PLUGIN_PROPERTIES);
                             break;
                         }
                         default:
