@@ -28,7 +28,7 @@ import org.jboss.as.ee.concurrent.handle.SetupContextHandle;
 import org.jboss.as.ejb3.context.CurrentInvocationContext;
 import org.jboss.invocation.InterceptorContext;
 
-import javax.enterprise.concurrent.ContextService;
+import jakarta.enterprise.concurrent.ContextService;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
@@ -74,6 +74,7 @@ public class EJBContextHandleFactory implements ContextHandleFactory {
 
     private static class EJBContextHandle implements SetupContextHandle, ResetContextHandle {
 
+        private static final long serialVersionUID = 9158258921823908698L;
         private final transient InterceptorContext interceptorContext;
 
         private EJBContextHandle() {
