@@ -23,27 +23,23 @@ package org.jboss.as.ejb3.iiop;
 
 
 import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.InvalidClassException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
-import javax.ejb.EJBHome;
-import javax.ejb.EJBMetaData;
+import jakarta.ejb.EJBHome;
+import jakarta.ejb.EJBMetaData;
 import javax.rmi.PortableRemoteObject;
+import jakarta.transaction.TransactionManager;
 
-import org.jacorb.ssl.SSLPolicyValue;
-import org.jacorb.ssl.SSLPolicyValueHelper;
-import org.jacorb.ssl.SSL_POLICY_TYPE;
 import org.jboss.as.ee.component.ComponentView;
-import org.jboss.as.ejb3.EjbLogger;
 import org.jboss.as.ejb3.component.EJBComponent;
-import org.jboss.as.ejb3.component.entity.EntityBeanComponent;
 import org.jboss.as.ejb3.component.stateless.StatelessSessionComponent;
 import org.jboss.as.ejb3.iiop.stub.DynamicStubFactoryFactory;
-import org.jboss.as.jacorb.csiv2.CSIv2Policy;
-import org.jboss.as.jacorb.rmi.ir.InterfaceRepository;
-import org.jboss.as.jacorb.rmi.marshal.strategy.SkeletonStrategy;
+import org.jboss.as.ejb3.logging.EjbLogger;
 import org.jboss.as.server.moduleservice.ServiceModuleLoader;
 import org.jboss.ejb.client.EJBHomeLocator;
 import org.jboss.ejb.client.EJBLocator;
@@ -53,11 +49,13 @@ import org.jboss.ejb.client.StatelessEJBLocator;
 import org.jboss.ejb.iiop.EJBMetaDataImplIIOP;
 import org.jboss.ejb.iiop.HandleImplIIOP;
 import org.jboss.ejb.iiop.HomeHandleImplIIOP;
+import org.jboss.marshalling.ClassResolver;
 import org.jboss.marshalling.Marshaller;
 import org.jboss.marshalling.MarshallerFactory;
 import org.jboss.marshalling.MarshallingConfiguration;
 import org.jboss.marshalling.ModularClassResolver;
 import org.jboss.marshalling.OutputStreamByteOutput;
+import org.jboss.marshalling.Unmarshaller;
 import org.jboss.marshalling.river.RiverMarshallerFactory;
 import org.jboss.metadata.ejb.jboss.IIOPMetaData;
 import org.jboss.metadata.ejb.jboss.IORSecurityConfigMetaData;
@@ -85,9 +83,14 @@ import org.omg.CosNaming.NamingContextPackage.NotFound;
 import org.omg.PortableServer.Current;
 import org.omg.PortableServer.CurrentHelper;
 import org.omg.PortableServer.POA;
+import org.wildfly.iiop.openjdk.csiv2.CSIv2Policy;
+import org.wildfly.iiop.openjdk.rmi.ir.InterfaceRepository;
+import org.wildfly.iiop.openjdk.rmi.marshal.strategy.SkeletonStrategy;
 import org.wildfly.security.manager.WildFlySecurityManager;
+import org.wildfly.transaction.client.ContextTransactionManager;
 
-import static org.jboss.as.ejb3.EjbMessages.MESSAGES;
+import com.arjuna.ats.jbossatx.jta.TransactionManagerService;
+import com.sun.corba.se.spi.extension.ZeroPortPolicy;
 
 /**
  * This is an IIOP "proxy factory" for <code>EJBHome</code>s and
@@ -99,6 +102,9 @@ import static org.jboss.as.ejb3.EjbMessages.MESSAGES;
  * <code>EjbHomeCorbaServant</code> for the bean's
  * <code>EJBHome</code> and an <code>EjbObjectCorbaServant</code> for the
  * bean's <code>EJBObject</code>s.
+ *
+ * NOTE: References in this document to Enterprise JavaBeans (EJB) refer to the Jakarta Enterprise Beans unless otherwise noted.
+ *
  */
 public class EjbIIOPService implements Service<EjbIIOPService> {
 
@@ -140,6 +146,11 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
      */
     private final InjectedValue<ServiceModuleLoader> serviceModuleLoaderInjectedValue = new InjectedValue<ServiceModuleLoader>();
 
+    /**
+     * The JTS underlying transaction manager service (not ContextTransactionManager)
+     */
+    private final InjectedValue<TransactionManagerService> transactionManagerInjectedValue = new InjectedValue<>();
+
     /**
      * Used for serializing EJB id's
      */
@@ -208,6 +219,11 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
      */
     private final InjectedValue<POA> irPoa = new InjectedValue<POA>();
 
+    /**
+     * Default IOR security config metadata (defined in the iiop subsystem).
+     */
+    private final InjectedValue<IORSecurityConfigMetaData> iorSecConfigMetaData = new InjectedValue<IORSecurityConfigMetaData>();
+
     /**
      * The IIOP metadata, configured in the assembly descriptor.
      */
@@ -231,15 +247,18 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
     }
 
 
+    @Override
     public synchronized void start(final StartContext startContext) throws StartException {
 
+
         try {
             final RiverMarshallerFactory factory = new RiverMarshallerFactory();
             final MarshallingConfiguration configuration = new MarshallingConfiguration();
-            configuration.setClassResolver(ModularClassResolver.getInstance(serviceModuleLoaderInjectedValue.getValue()));
+            configuration.setClassResolver(new FilteringClassResolver(ModularClassResolver.getInstance(serviceModuleLoaderInjectedValue.getValue())));
             this.configuration = configuration;
             this.factory = factory;
-
+            final TransactionManager jtsTransactionManager = transactionManagerInjectedValue.getValue().getTransactionManager();
+            assert ! (jtsTransactionManager instanceof ContextTransactionManager);
 
             // Should create a CORBA interface repository?
             final boolean interfaceRepositorySupported = false;
@@ -261,6 +280,8 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
                 name = component.getComponentName();
             }
             name = name.replace(".", "_");
+            EjbLogger.DEPLOYMENT_LOGGER.iiopBindings(component.getComponentName(), component.getModuleName(),
+                    name);
 
             final ORB orb = this.orb.getValue();
             if (interfaceRepositorySupported) {
@@ -273,8 +294,8 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
                 EjbLogger.ROOT_LOGGER.cobraInterfaceRepository(name, orb.object_to_string(iri.getReference()));
             }
 
-            IORSecurityConfigMetaData iorSecurityConfigMetaData = null;
-            if (this.iiopMetaData != null)
+            IORSecurityConfigMetaData iorSecurityConfigMetaData = this.iorSecConfigMetaData.getOptionalValue();
+            if (this.iiopMetaData != null && this.iiopMetaData.getIorSecurityConfigMetaData() != null)
                 iorSecurityConfigMetaData = this.iiopMetaData.getIorSecurityConfigMetaData();
 
             // Create security policies if security metadata has been provided.
@@ -285,27 +306,25 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
                 final Any secPolicy = orb.create_any();
                 secPolicy.insert_Value(iorSecurityConfigMetaData);
                 Policy csiv2Policy = orb.create_policy(CSIv2Policy.TYPE, secPolicy);
+
                 policyList.add(csiv2Policy);
 
-                // Create SSLPolicy (SSL_REQUIRED ensures home and remote IORs will have port 0 in the primary address).
+                //  Add ZeroPortPolicy if ssl is required (it ensures home and remote IORs will have port 0 in the primary address).
                 boolean sslRequired = false;
-                if (iorSecurityConfigMetaData != null) {
+                if (iorSecurityConfigMetaData != null && iorSecurityConfigMetaData.getTransportConfig() != null) {
                     IORTransportConfigMetaData tc = iorSecurityConfigMetaData.getTransportConfig();
                     sslRequired = IORTransportConfigMetaData.INTEGRITY_REQUIRED.equals(tc.getIntegrity())
                             || IORTransportConfigMetaData.CONFIDENTIALITY_REQUIRED.equals(tc.getConfidentiality())
                             || IORTransportConfigMetaData.ESTABLISH_TRUST_IN_CLIENT_REQUIRED.equals(tc.getEstablishTrustInClient());
                 }
-                final Any sslPolicyValue = orb.create_any();
-                SSLPolicyValueHelper.insert(sslPolicyValue, (sslRequired) ? SSLPolicyValue.SSL_REQUIRED : SSLPolicyValue.SSL_NOT_REQUIRED);
-                Policy sslPolicy = orb.create_policy(SSL_POLICY_TYPE.value, sslPolicyValue);
-                policyList.add(sslPolicy);
-
-                EjbLogger.ROOT_LOGGER.debug("container's SSL policy: " + sslPolicy);
+                if(sslRequired){
+                    policyList.add(ZeroPortPolicy.getPolicy());
+                }
             }
 
-            String securityDomain = "CORBA_REMOTE"; //TODO: what should this default to
+            String securityDomainName = "CORBA_REMOTE"; //TODO: what should this default to
             if (component.getSecurityMetaData() != null) {
-                securityDomain = component.getSecurityMetaData().getSecurityDomain();
+                securityDomainName = component.getSecurityMetaData().getSecurityDomainName();
             }
 
             Policy[] policies = policyList.toArray(new Policy[policyList.size()]);
@@ -322,7 +341,8 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
 
             // Instantiate home servant, bind it to the servant registry, and create CORBA reference to the EJBHome.
             final EjbCorbaServant homeServant = new EjbCorbaServant(poaCurrent, homeMethodMap, homeRepositoryIds, homeInterfaceDef,
-                    orb, homeView.getValue(), factory, configuration, component.getTransactionManager(), module.getClassLoader(), true, securityDomain);
+                    orb, homeView.getValue(), factory, configuration, jtsTransactionManager, module.getClassLoader(), true, securityDomainName,
+                    component.getSecurityDomain());
 
             homeServantRegistry = poaRegistry.getValue().getRegistryWithPersistentPOAPerServant();
             ReferenceFactory homeReferenceFactory = homeServantRegistry.bind(homeServantName(name), homeServant, policies);
@@ -335,25 +355,17 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
             final HomeHandleImplIIOP homeHandle = new HomeHandleImplIIOP(orb.object_to_string(corbaRef));
             homeServant.setHomeHandle(homeHandle);
 
-            // Initialize beanPOA and create metadata depending on the kind of bean
-            if (component instanceof EntityBeanComponent) {
-
-                // This is an entity bean (lifespan: persistent)
-                beanServantRegistry = poaRegistry.getValue().getRegistryWithPersistentPOAPerServant();
-                final EntityBeanComponent entityBeanComponent = (EntityBeanComponent) component;
-                final Class pkClass = entityBeanComponent.getPrimaryKeyClass();
-                ejbMetaData = new EJBMetaDataImplIIOP(entityBeanComponent.getRemoteClass(), entityBeanComponent.getHomeClass(), pkClass, false, false, homeHandle);
+            // Initialize beanPOA and create metadata
+            // This is a session bean (lifespan: transient)
+            beanServantRegistry = poaRegistry.getValue().getRegistryWithTransientPOAPerServant();
+            if (component instanceof StatelessSessionComponent) {
+                // Stateless session bean
+                ejbMetaData = new EJBMetaDataImplIIOP(remoteView.getValue().getViewClass(), homeView.getValue().getViewClass(), null, true, true, homeHandle);
             } else {
-                // This is a session bean (lifespan: transient)
-                beanServantRegistry = poaRegistry.getValue().getRegistryWithTransientPOAPerServant();
-                if (component instanceof StatelessSessionComponent) {
-                    // Stateless session bean
-                    ejbMetaData = new EJBMetaDataImplIIOP(remoteView.getValue().getViewClass(), homeView.getValue().getViewClass(), null, true, true, homeHandle);
-                } else {
-                    // Stateful session bean
-                    ejbMetaData = new EJBMetaDataImplIIOP(remoteView.getValue().getViewClass(), homeView.getValue().getViewClass(), null, true, false, homeHandle);
-                }
+                // Stateful session bean
+                ejbMetaData = new EJBMetaDataImplIIOP(remoteView.getValue().getViewClass(), homeView.getValue().getViewClass(), null, true, false, homeHandle);
             }
+
             homeServant.setEjbMetaData(ejbMetaData);
 
             // If there is an interface repository, then get the beanInterfaceDef from the IR
@@ -365,13 +377,13 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
 
             // Instantiate the ejb object servant and bind it to the servant registry.
             final EjbCorbaServant beanServant = new EjbCorbaServant(poaCurrent, beanMethodMap, beanRepositoryIds,
-                    beanInterfaceDef, orb, remoteView.getValue(), factory, configuration, component.getTransactionManager(),
-                    module.getClassLoader(), false, securityDomain);
+                    beanInterfaceDef, orb, remoteView.getValue(), factory, configuration, jtsTransactionManager,
+                    module.getClassLoader(), false, securityDomainName, component.getSecurityDomain());
             beanReferenceFactory = beanServantRegistry.bind(beanServantName(name), beanServant, policies);
 
             // Register bean home in local CORBA naming context
             rebind(corbaNamingContext.getValue(), name, corbaRef);
-            EjbLogger.ROOT_LOGGER.debug("Home IOR for " + component.getComponentName() + " bound to " + this.name + " in CORBA naming service");
+            EjbLogger.ROOT_LOGGER.debugf("Home IOR for %s bound to %s in CORBA naming service", component.getComponentName(), this.name);
 
             //now eagerly force stub creation, so de-serialization of stubs will work correctly
             final ClassLoader cl = WildFlySecurityManager.getCurrentContextClassLoaderPrivileged();
@@ -380,12 +392,12 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
                 try {
                     DynamicStubFactoryFactory.makeStubClass(homeView.getValue().getViewClass());
                 } catch (Exception e) {
-                    EjbLogger.EJB3_LOGGER.dynamicStubCreationFailed(homeView.getValue().getViewClass().getName(), e);
+                    EjbLogger.ROOT_LOGGER.dynamicStubCreationFailed(homeView.getValue().getViewClass().getName(), e);
                 }
                 try {
                     DynamicStubFactoryFactory.makeStubClass(remoteView.getValue().getViewClass());
                 } catch (Exception e) {
-                    EjbLogger.EJB3_LOGGER.dynamicStubCreationFailed(remoteView.getValue().getViewClass().getName(), e);
+                    EjbLogger.ROOT_LOGGER.dynamicStubCreationFailed(remoteView.getValue().getViewClass().getName(), e);
                 }
             } finally {
                 WildFlySecurityManager.setCurrentContextClassLoaderPrivileged(cl);
@@ -452,26 +464,28 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
                 } else if (locator instanceof StatelessEJBLocator) {
                     return beanReferenceFactory.createReference(beanRepositoryIds[0]);
                 } else if (locator instanceof StatefulEJBLocator) {
-                    final Marshaller marshaller = factory.createMarshaller(configuration);
-                    final ByteArrayOutputStream stream = new ByteArrayOutputStream();
-                    marshaller.start(new OutputStreamByteOutput(stream));
-                    marshaller.writeObject(((StatefulEJBLocator) locator).getSessionId());
-                    marshaller.finish();
-                    return beanReferenceFactory.createReferenceWithId(stream.toByteArray(), beanRepositoryIds[0]);
+                    try (final Marshaller marshaller = factory.createMarshaller(configuration)) {
+                        final ByteArrayOutputStream stream = new ByteArrayOutputStream();
+                        marshaller.start(new OutputStreamByteOutput(stream));
+                        marshaller.writeObject(((StatefulEJBLocator<?>) locator).getSessionId());
+                        marshaller.flush();
+                        return beanReferenceFactory.createReferenceWithId(stream.toByteArray(), beanRepositoryIds[0]);
+                    }
                 } else if (locator instanceof EntityEJBLocator) {
-                    final Marshaller marshaller = factory.createMarshaller(configuration);
-                    final ByteArrayOutputStream stream = new ByteArrayOutputStream();
-                    marshaller.start(new OutputStreamByteOutput(stream));
-                    marshaller.writeObject(((EntityEJBLocator) locator).getPrimaryKey());
-                    marshaller.finish();
-                    return beanReferenceFactory.createReferenceWithId(stream.toByteArray(), beanRepositoryIds[0]);
+                    try (final Marshaller marshaller = factory.createMarshaller(configuration)) {
+                        final ByteArrayOutputStream stream = new ByteArrayOutputStream();
+                        marshaller.start(new OutputStreamByteOutput(stream));
+                        marshaller.writeObject(((EntityEJBLocator<?>) locator).getPrimaryKey());
+                        marshaller.flush();
+                        return beanReferenceFactory.createReferenceWithId(stream.toByteArray(), beanRepositoryIds[0]);
+                    }
                 }
-                throw MESSAGES.unknownEJBLocatorType(locator);
+                throw EjbLogger.ROOT_LOGGER.unknownEJBLocatorType(locator);
             } else {
-                throw MESSAGES.incorrectEJBLocatorForBean(locator, ejbComponent.getComponentName());
+                throw EjbLogger.ROOT_LOGGER.incorrectEJBLocatorForBean(locator, ejbComponent.getComponentName());
             }
         } catch (Exception e) {
-            throw MESSAGES.couldNotCreateCorbaObject(e, locator);
+            throw EjbLogger.ROOT_LOGGER.couldNotCreateCorbaObject(e, locator);
         }
     }
 
@@ -481,7 +495,7 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
      * @param locator The locator to get the handle for
      * @return The {@link org.jboss.ejb.client.EJBHandle} or {@link org.jboss.ejb.client.EJBHomeHandle}
      */
-    public Object handleForLocator(final EJBLocator locator) {
+    public Object handleForLocator(final EJBLocator<?> locator) {
         final org.omg.CORBA.Object reference = referenceForLocator(locator);
         if(locator instanceof EJBHomeLocator) {
             return new HomeHandleImplIIOP(orb.getValue().object_to_string(reference));
@@ -505,6 +519,7 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
         final NameComponent[] name = ctx.to_name(strName);
         NamingContext intermediateCtx = ctx;
 
+
         for (int i = 0; i < name.length - 1; i++) {
             final NameComponent[] relativeName = new NameComponent[]{name[i]};
             try {
@@ -569,4 +584,66 @@ public class EjbIIOPService implements Service<EjbIIOPService> {
     public InjectedValue<ServiceModuleLoader> getServiceModuleLoaderInjectedValue() {
         return serviceModuleLoaderInjectedValue;
     }
+
+    public InjectedValue<IORSecurityConfigMetaData> getIORSecConfigMetaDataInjectedValue() {
+        return iorSecConfigMetaData;
+    }
+
+    public InjectedValue<TransactionManagerService> getTransactionManagerInjectedValue() {
+        return transactionManagerInjectedValue;
+    }
+
+    // ModularClassResolver is final so we have to wrap it
+    private static final class FilteringClassResolver implements ClassResolver {
+        private final ClassResolver delegate;
+
+        private FilteringClassResolver(ClassResolver delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public void annotateClass(Marshaller marshaller, Class<?> clazz) throws IOException {
+            delegate.annotateClass(marshaller, clazz);
+        }
+
+        @Override
+        public void annotateProxyClass(Marshaller marshaller, Class<?> proxyClass) throws IOException {
+            delegate.annotateProxyClass(marshaller, proxyClass);
+        }
+
+        @Override
+        public String getClassName(Class<?> clazz) throws IOException {
+            return delegate.getClassName(clazz);
+        }
+
+        @Override
+        public String[] getProxyInterfaces(Class<?> proxyClass) throws IOException {
+            return delegate.getProxyInterfaces(proxyClass);
+        }
+
+        @Override
+        public Class<?> resolveClass(Unmarshaller unmarshaller, String name, long serialVersionUID) throws IOException, ClassNotFoundException {
+            checkFilter(name);
+            return delegate.resolveClass(unmarshaller, name, serialVersionUID);
+        }
+
+        @Override
+        public Class<?> resolveProxyClass(Unmarshaller unmarshaller, String[] interfaces) throws IOException, ClassNotFoundException {
+            for (String name : interfaces) {
+                checkFilter(name);
+            }
+            return delegate.resolveProxyClass(unmarshaller, interfaces);
+        }
+
+        private void checkFilter(String className) throws InvalidClassException {
+            // We only resolve org.jboss.ejb.client.SessionId impls in its package.
+            // The SessionId abstract class is public but its constructor is package
+            // protected, so the only valid impls are in that package.
+            // The JBoss Marshalling 'Configuration' object instances that use this
+            // ClassResolver are only meant to marshal/unmarshal SessionId objects.
+            if (!className.startsWith("org.jboss.ejb.client.")) {
+                throw EjbLogger.REMOTE_LOGGER.cannotResolveFilteredClass(className);
+            }
+        }
+    }
 }
