@@ -47,7 +47,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.CountDownLatch;
 
-import javax.resource.spi.TransactionSupport;
+import jakarta.resource.spi.TransactionSupport;
 import org.apache.activemq.artemis.api.core.DiscoveryGroupConfiguration;
 import org.apache.activemq.artemis.api.core.TransportConfiguration;
 import org.apache.activemq.artemis.ra.ActiveMQRAConnectionFactoryImpl;
@@ -59,7 +59,6 @@ import org.jboss.as.controller.PathAddress;
 import org.jboss.as.controller.PathElement;
 import org.jboss.as.controller.descriptions.ModelDescriptionConstants;
 import org.jboss.as.ee.component.EEModuleDescription;
-import org.jboss.as.ee.component.InjectionSource;
 import org.jboss.as.ee.resource.definition.ResourceDefinitionInjectionSource;
 import org.jboss.as.naming.ContextListAndJndiViewManagedReferenceFactory;
 import org.jboss.as.naming.ManagedReference;
@@ -104,7 +103,7 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
     /*
     String description() default "";
     String name();
-    String interfaceName() default "javax.jms.ConnectionFactory";
+    String interfaceName() default "jakarta.jms.ConnectionFactory";
     String className() default "";
     String resourceAdapter() default "";
     String user() default "";
@@ -260,6 +259,7 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
         if(external) {
             serverName = null;
             Set<String> connectorsSocketBindings = new HashSet<>();
+            Set<String> sslContextNames = new HashSet<>();
             ExternalBrokerConfigurationService configuration = (ExternalBrokerConfigurationService)deploymentUnit.getServiceRegistry().getRequiredService(MessagingSubsystemRootResourceDefinition.CONFIGURATION_CAPABILITY.getCapabilityServiceName()).getService().getValue();
             TransportConfiguration[] tcs = new TransportConfiguration[connectors.size()];
             for (int i = 0; i < tcs.length; i++) {
@@ -267,6 +267,9 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
                 if (tcs[i].getParams().containsKey(ModelDescriptionConstants.SOCKET_BINDING)) {
                     connectorsSocketBindings.add(tcs[i].getParams().get(ModelDescriptionConstants.SOCKET_BINDING).toString());
                 }
+                if (tcs[i].getParams().containsKey(ModelDescriptionConstants.SSL_CONTEXT)) {
+                    sslContextNames.add(tcs[i].getParams().get(ModelDescriptionConstants.SSL_CONTEXT).toString());
+                }
             }
             DiscoveryGroupConfiguration discoveryGroupConfiguration = null;
             if(discoveryGroupName != null) {
@@ -274,14 +277,17 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
             }
             if (connectors.isEmpty() && discoveryGroupConfiguration == null) {
                 tcs = getExternalPooledConnectionFactory(resourceAdapter, deploymentUnit.getServiceRegistry()).getConnectors();
-                for(int i = 0 ; i < tcs.length; i++) {
-                 if(tcs[i].getParams().containsKey(ModelDescriptionConstants.SOCKET_BINDING)) {
-                    connectorsSocketBindings.add(tcs[i].getParams().get(ModelDescriptionConstants.SOCKET_BINDING).toString());
-                 }
-             }
+                for (int i = 0; i < tcs.length; i++) {
+                    if (tcs[i].getParams().containsKey(ModelDescriptionConstants.SOCKET_BINDING)) {
+                        connectorsSocketBindings.add(tcs[i].getParams().get(ModelDescriptionConstants.SOCKET_BINDING).toString());
+                    }
+                    if (tcs[i].getParams().containsKey(ModelDescriptionConstants.SSL_CONTEXT)) {
+                        sslContextNames.add(tcs[i].getParams().get(ModelDescriptionConstants.SSL_CONTEXT).toString());
+                    }
+                }
             }
             ExternalPooledConnectionFactoryService.installService(serviceTarget, configuration, pcfName, tcs, discoveryGroupConfiguration,
-                    connectorsSocketBindings, null, jgroupsChannelName, adapterParams, bindInfo, Collections.emptyList(),
+                    connectorsSocketBindings, sslContextNames, null, jgroupsChannelName, adapterParams, bindInfo, Collections.emptyList(),
                     txSupport, minPoolSize, maxPoolSize, managedConnectionPoolClassName, enlistmentTrace, deploymentUnit.getAttachment(CAPABILITY_SERVICE_SUPPORT));
         } else {
             serverName = getActiveMQServerName(properties);
@@ -308,7 +314,11 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
             registration = PathAddress.pathAddress(serverElement, pcfPath);
         }
         MessagingXmlInstallDeploymentUnitProcessor.createDeploymentSubModel(registration, deploymentUnit);
-        PooledConnectionFactoryConfigurationRuntimeHandler.INSTANCE.registerResource(serverName, managementName, model);
+        if(external) {
+            PooledConnectionFactoryConfigurationRuntimeHandler.EXTERNAL_INSTANCE.registerResource(serverName, managementName, model);
+        } else {
+            PooledConnectionFactoryConfigurationRuntimeHandler.INSTANCE.registerResource(serverName, managementName, model);
+        }
     }
 
     private List<String> getConnectors(Map<String, String> props) {
@@ -332,7 +342,7 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
     }
 
     void clearUnknownProperties(final Map<String, String> props) {
-        Set<String> attributeNames = PooledConnectionFactoryDefinition.getAttributesMap().keySet();
+        Map<String, ConnectionFactoryAttribute> attributesMap = PooledConnectionFactoryDefinition.getAttributesMap();
 
         final Iterator<Map.Entry<String, String>> it = props.entrySet().iterator();
         while (it.hasNext()) {
@@ -340,21 +350,21 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
             String value = entry.getKey();
             if (value == null || "".equals(value)) {
                 it.remove();
-            } else if (!attributeNames.contains(entry.getKey())) {
+            } else if (!attributesMap.containsKey(entry.getKey())) {
                 MessagingLogger.ROOT_LOGGER.unknownPooledConnectionFactoryAttribute(entry.getKey());
                 it.remove();
             }
         }
     }
 
-    private static String uniqueName(InjectionSource.ResolutionContext context, final String jndiName) {
+    private static String uniqueName(ResolutionContext context, final String jndiName) {
         StringBuilder uniqueName = new StringBuilder();
         return uniqueName.append(context.getApplicationName()).append("_")
                 .append(managementName(context, jndiName))
                 .toString();
     }
 
-    private static String managementName(InjectionSource.ResolutionContext context, final String jndiName) {
+    private static String managementName(ResolutionContext context, final String jndiName) {
         StringBuilder uniqueName = new StringBuilder();
         uniqueName.append(context.getModuleName()).append("_");
         if (context.getComponentName() != null) {
@@ -411,13 +421,13 @@ public class JMSConnectionFactoryDefinitionInjectionSource extends ResourceDefin
             return false;
         }
         //let's look into the external-pooled-connection-factory
-        ServiceName pcfName = JMSServices.getPooledConnectionFactoryBaseServiceName(MessagingServices.getActiveMQServiceName("")).append(resourceAdapter);
+        ServiceName pcfName = JMSServices.getPooledConnectionFactoryBaseServiceName(MessagingServices.getActiveMQServiceName()).append(resourceAdapter);
         return serviceRegistry.getServiceNames().contains(pcfName);
     }
 
     private static ExternalPooledConnectionFactoryService getExternalPooledConnectionFactory(String resourceAdapter, ServiceRegistry serviceRegistry) {
         //let's look into the external-pooled-connection-factory
-        ServiceName pcfName = JMSServices.getPooledConnectionFactoryBaseServiceName(MessagingServices.getActiveMQServiceName("")).append(resourceAdapter);
+        ServiceName pcfName = JMSServices.getPooledConnectionFactoryBaseServiceName(MessagingServices.getActiveMQServiceName()).append(resourceAdapter);
         return (ExternalPooledConnectionFactoryService) serviceRegistry.getService(pcfName).getValue();
     }
 
