@@ -36,6 +36,7 @@ import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -51,6 +52,7 @@ import org.jboss.as.test.integration.domain.management.util.DomainLifecycleUtil;
 import org.jboss.as.test.integration.domain.management.util.DomainTestUtils;
 import org.jboss.as.test.integration.domain.management.util.WildFlyManagedConfiguration;
 import org.jboss.as.test.integration.management.util.MgmtOperationException;
+import org.jboss.as.test.shared.util.AssumeTestGroupUtil;
 import org.jboss.dmr.ModelNode;
 import org.jboss.dmr.Property;
 import org.jboss.logging.Logger;
@@ -81,13 +83,14 @@ import org.junit.Test;
  */
 public class HostExcludesTestCase extends BuildConfigurationTestBase {
 
-    private static DomainLifecycleUtil masterUtils;
-    private static DomainClient masterClient;
-    private static WildFlyManagedConfiguration masterConfig;
+    private static DomainLifecycleUtil primaryUtils;
+    private static DomainClient primaryClient;
+    private static WildFlyManagedConfiguration primaryConfig;
     private static final  BiFunction<Set<String>, Set<String>, Set<String>> diff = (a, b) -> a.stream().filter(e -> !b.contains(e)).collect(Collectors.toSet());
-    private final boolean isEeGalleonPack = "ee-".equals(System.getProperty("testsuite.default.build.project.prefix"));
+    private final boolean isFullDistribution = AssumeTestGroupUtil.isFullDistribution();
+    private final boolean isPreviewGalleonPack = AssumeTestGroupUtil.isWildFlyPreview();
 
-    private static final String MAJOR = "25.";
+    private static final String MAJOR = "28.";
 
     /**
      * Maintains the list of expected extensions for each host-exclude name for previous releases.
@@ -96,7 +99,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
      * This must be corrected on each new host-exclude id added on the current release.
      */
     private enum ExtensionConf {
-        WILDFLY_10_0("WildFly10.0", Arrays.asList(
+        WILDFLY_10_0("WildFly10.0", null, Arrays.asList(
                 "org.jboss.as.appclient",
                 "org.jboss.as.clustering.infinispan",
                 "org.jboss.as.clustering.jgroups",
@@ -129,6 +132,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
                 "org.jboss.as.webservices",
                 "org.jboss.as.weld",
                 "org.jboss.as.xts",
+                "org.keycloak.keycloak-adapter-subsystem",
                 "org.wildfly.extension.batch.jberet",
                 "org.wildfly.extension.bean-validation",
                 "org.wildfly.extension.clustering.singleton",
@@ -141,110 +145,157 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
                 "org.wildfly.extension.security.manager",
                 "org.wildfly.extension.undertow",
                 "org.wildfly.iiop-openjdk"
-        )),
-        WILDFLY_10_1("WildFly10.1", WILDFLY_10_0),
+        ), false),
+        WILDFLY_10_1("WildFly10.1", WILDFLY_10_0, false),
         WILDFLY_11_0("WildFly11.0", WILDFLY_10_1, Arrays.asList(
                 "org.wildfly.extension.core-management",
                 "org.wildfly.extension.discovery",
                 "org.wildfly.extension.elytron"
-        )),
-        WILDFLY_12_0("WildFly12.0", WILDFLY_11_0),
-        WILDFLY_13_0("WildFly13.0", WILDFLY_12_0, Arrays.asList(
+        ), false),
+        WILDFLY_12_0("WildFly12.0", WILDFLY_11_0, false),
+        WILDFLY_13_0("WildFly13.0", WILDFLY_12_0, List.of(
                 "org.wildfly.extension.ee-security"
-        )),
+        ), false),
         WILDFLY_14_0("WildFly14.0", WILDFLY_13_0, Arrays.asList(
                 "org.wildfly.extension.datasources-agroal",
                 "org.wildfly.extension.microprofile.config-smallrye",
                 "org.wildfly.extension.microprofile.health-smallrye",
                 "org.wildfly.extension.microprofile.opentracing-smallrye"
-        )),
-        WILDFLY_15_0("WildFly15.0", WILDFLY_14_0, Arrays.asList(
+        ), false),
+        WILDFLY_15_0("WildFly15.0", WILDFLY_14_0, List.of(
                 "org.wildfly.extension.microprofile.metrics-smallrye"
-        )),
-        WILDFLY_16_0("WildFly16.0", WILDFLY_15_0, Arrays.asList(
+        ), false),
+        WILDFLY_16_0("WildFly16.0", WILDFLY_15_0, List.of(
                 // This extension was added in WF17, however we add it here because WF16/WF17/WF18 use the same management
                 // kernel API, which is 10.0.0. Adding a host-exclusion for this extension on WF16 could affect to WF17/WF18
                 // We decided to add the host-exclusion only for WF15 and below. It means potentially a DC running on WF17
-                // with an WF16 as slave will not exclude this extension. It is not a problem at all since mixed domains in
+                // with an WF16 as secondary will not exclude this extension. It is not a problem at all since mixed domains in
                 // WildFly is not supported.
                 "org.wildfly.extension.clustering.web"
-        )),
-        WILDFLY_17_0("WildFly17.0", WILDFLY_16_0),
-        WILDFLY_18_0("WildFly18.0", WILDFLY_17_0),
+        ), false),
+        WILDFLY_17_0("WildFly17.0", WILDFLY_16_0, false),
+        WILDFLY_18_0("WildFly18.0", WILDFLY_17_0, false),
         WILDFLY_19_0("WildFly19.0", WILDFLY_18_0, Arrays.asList(
                 "org.wildfly.extension.microprofile.fault-tolerance-smallrye",
                 "org.wildfly.extension.microprofile.jwt-smallrye",
                 "org.wildfly.extension.microprofile.openapi-smallrye"
-        )),
-        WILDFLY_20_0("WildFly20.0", WILDFLY_19_0),
-        WILDFLY_21_0("WildFly21.0", WILDFLY_20_0),
+        ), false),
+        WILDFLY_20_0("WildFly20.0", WILDFLY_19_0, false),
+        WILDFLY_21_0("WildFly21.0", WILDFLY_20_0, false),
         WILDFLY_22_0("WildFly22.0", WILDFLY_21_0, Arrays.asList(
                 "org.wildfly.extension.health",
                 "org.wildfly.extension.metrics"
-        )),
+        ), false),
         WILDFLY_23_0("WildFly23.0", WILDFLY_22_0, Arrays.asList(
                 "org.wildfly.extension.microprofile.reactive-messaging-smallrye",
                 "org.wildfly.extension.microprofile.reactive-streams-operators-smallrye"
-        )),
-        WILDFLY_24_0("WildFly24.0", WILDFLY_23_0),
-        // If an extension is added to this enum, also check if it is supplied only by wildfly-galleon-pack. If so, add it also
-        // to the internal mpExtensions Set defined on this class.
-        CURRENT(MAJOR, WILDFLY_24_0, Arrays.asList(
+        ), false),
+        WILDFLY_24_0("WildFly24.0", WILDFLY_23_0, true),
+        WILDFLY_25_0("WildFly25.0", WILDFLY_24_0, Arrays.asList(
                 "org.wildfly.extension.elytron-oidc-client",
                 "org.wildfly.extension.opentelemetry"
-        ));
+        ), true),
+        WILDFLY_26_0("WildFly26.0", WILDFLY_25_0, null, Arrays.asList(
+                "org.jboss.as.cmp",
+                "org.jboss.as.jaxr",
+                "org.jboss.as.configadmin"
+        ), true),
+        WILDFLY_27_0("WildFly27.0", WILDFLY_26_0, Arrays.asList(
+                "org.wildfly.extension.clustering.ejb",
+                "org.wildfly.extension.datasources-agroal"
+        ), true),
+        // If an extension is added to this enum, also check if it is supplied only by wildfly-galleon-pack. If so, add it also
+        // to the internal mpExtensions Set defined on this class.
+        // Don't add here extensions supplied only by the wildfly-preview-feature-pack because we are not tracking different releases
+        // of wildfly preview. In such a case, add them to previewExtensions set defined below.
+        CURRENT(MAJOR, WILDFLY_27_0, Arrays.asList(
+            "org.wildfly.extension.micrometer",
+            "org.wildfly.extension.microprofile.lra-coordinator",
+            "org.wildfly.extension.microprofile.lra-participant",
+            "org.wildfly.extension.microprofile.telemetry"
+        ), getCurrentRemovedExtensions(), true);
+
+        private static List<String> getCurrentRemovedExtensions() {
+            // TODO If we decide to remove these modules from WFP, uncomment this.
+            // See https://issues.redhat.com/browse/WFLY-16686
+            /*
+            if (AssumeTestGroupUtil.isWildFlyPreview()) {
+                return Arrays.asList(
+                        "org.jboss.as.messaging",
+                        "org.jboss.as.jacorb",
+                        "org.jboss.as.jsr77",
+                        "org.jboss.as.web",
+                        "org.wildfly.extension.picketlink",
+                        "org.jboss.as.security"
+                        );
+            }
+            */
+            return Collections.emptyList();
+        }
 
         private final String name;
         private final Set<String> extensions = new HashSet<>();
+        private final Set<String> removed = new HashSet<>();
         private static final Map<String, ExtensionConf> MAP;
         private final boolean modified;
+        private final boolean supported;
 
-        // List of extensions added only by the wildfly-galleon-pack
-        private Set<String> mpExtensions = new HashSet<>(Arrays.asList(
+        // List of extensions added by the wildfly-galleon-pack
+        private final Set<String> mpExtensions = new HashSet<>(Arrays.asList(
+                "org.wildfly.extension.micrometer",
                 "org.wildfly.extension.microprofile.config-smallrye",
                 "org.wildfly.extension.microprofile.health-smallrye",
                 "org.wildfly.extension.microprofile.metrics-smallrye",
-                "org.wildfly.extension.microprofile.opentracing-smallrye",
                 "org.wildfly.extension.microprofile.fault-tolerance-smallrye",
                 "org.wildfly.extension.microprofile.jwt-smallrye",
                 "org.wildfly.extension.microprofile.openapi-smallrye",
                 "org.wildfly.extension.microprofile.reactive-messaging-smallrye",
                 "org.wildfly.extension.microprofile.reactive-streams-operators-smallrye",
-                "org.wildfly.extension.elytron-oidc-client"
+                "org.wildfly.extension.microprofile.lra-coordinator",
+                "org.wildfly.extension.microprofile.lra-participant",
+                "org.wildfly.extension.microprofile.telemetry"
         ));
 
-        ExtensionConf(String name, List<String> addedExtensions) {
-            this(name, null, addedExtensions, null);
+        // List of extensions added only by the WildFly Preview
+        // We do not track changes between different versions of WildFly Preview.
+        // From the point of view of this test, all extensions added in WildFly Preview are always extensions
+        // added in the latest release of WildFly Preview. It is out of the scope of Host Exclusion test
+        // to compute on which WildFly Preview was added such a new extension and track the Host Exclusions between
+        // different WildFly Preview releases.
+        private final Set<String> previewExtensions = new HashSet<>(Arrays.asList(
+        ));
+
+        ExtensionConf(String name, ExtensionConf parent, boolean supported) {
+            this(name, parent, null, null, supported);
         }
 
-        ExtensionConf(String name, ExtensionConf parent) {
-            this(name, parent, null, null);
-        }
-
-        ExtensionConf(String name, ExtensionConf parent, List<String> addedExtensions) {
-            this(name, parent, addedExtensions, null);
+        ExtensionConf(String name, ExtensionConf parent, List<String> addedExtensions, boolean supported) {
+            this(name, parent, addedExtensions, null, supported);
         }
 
         /**
          * Main constructor
-         *
-         * @param name Host exclude name to define
+         *  @param name Host exclude name to define
          * @param parent A parent extension definition
          * @param addedExtensions Extensions added on the server release referred by this host exclude name
          * @param removedExtensions Extensions removed on the server release referred by this host exclude name
+         * @param supported whether the given release is supported as a secondary host in a mixed domain
          */
-        ExtensionConf(String name, ExtensionConf parent, List<String> addedExtensions, List<String> removedExtensions) {
+        ExtensionConf(String name, ExtensionConf parent, List<String> addedExtensions, List<String> removedExtensions, boolean supported) {
             this.name = name;
             this.modified = (addedExtensions != null && !addedExtensions.isEmpty()) || (removedExtensions != null && !removedExtensions.isEmpty());
             if (addedExtensions != null) {
                 this.extensions.addAll(addedExtensions);
             }
-            if (parent != null && parent.extensions != null) {
+            if (parent != null) {
                 this.extensions.addAll(parent.extensions);
+                this.removed.addAll(parent.removed);
             }
             if (removedExtensions != null) {
                 this.extensions.removeAll(removedExtensions);
+                this.removed.addAll(removedExtensions);
             }
+            this.supported = supported;
         }
 
         static {
@@ -260,8 +311,15 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
             return MAP.get(name);
         }
 
-        public Set<String> getExtensions(boolean isEeGalleonPack) {
-            if (isEeGalleonPack) {
+        public Set<String> getRemovedExtensions() {
+            return removed;
+        }
+
+        public Set<String> getExtensions(boolean isFullDistribution, boolean isPreview) {
+            if (this.name.equals(MAJOR) && isPreview) {
+                this.extensions.addAll(previewExtensions);
+            }
+            if (!isFullDistribution && !isPreview) {
                 return diff.apply(extensions, mpExtensions);
             }
             return extensions;
@@ -271,6 +329,10 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
             return modified;
         }
 
+        public boolean isSupported() {
+            return supported;
+        }
+
         public String getName() {
             return name;
         }
@@ -278,16 +340,16 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
 
     @BeforeClass
     public static void setUp() throws IOException {
-        masterConfig = createConfiguration("domain.xml", "host-master.xml", HostExcludesTestCase.class.getSimpleName());
-        masterUtils = new DomainLifecycleUtil(masterConfig);
-        masterUtils.start();
-        masterClient = masterUtils.getDomainClient();
+        primaryConfig = createConfiguration("domain.xml", "host-primary.xml", HostExcludesTestCase.class.getSimpleName());
+        primaryUtils = new DomainLifecycleUtil(primaryConfig);
+        primaryUtils.start();
+        primaryClient = primaryUtils.getDomainClient();
     }
 
     @AfterClass
     public static void tearDown() {
-        if (masterUtils != null) {
-            masterUtils.stop();
+        if (primaryUtils != null) {
+            primaryUtils.stop();
         }
     }
 
@@ -298,7 +360,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
         ModelNode op = Util.getEmptyOperation(READ_CHILDREN_RESOURCES_OPERATION, null);
         op.get(CHILD_TYPE).set(EXTENSION);
 
-        ModelNode result = DomainTestUtils.executeForResult(op, masterClient);
+        ModelNode result = DomainTestUtils.executeForResult(op, primaryClient);
 
         Set<String> currentExtensions = new HashSet<>();
         for (Property prop : result.asPropertyList()) {
@@ -313,7 +375,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
         }
 
         // Check that the list of all available extensions is in the ExtensionConf.CURRENT configuration
-        Set<String> current = ExtensionConf.forName(MAJOR).getExtensions(isEeGalleonPack);
+        Set<String> current = ExtensionConf.forName(MAJOR).getExtensions(isFullDistribution, isPreviewGalleonPack);
         if (!current.equals(availableExtensions)) {
             Set<String> extensionsAdded = diff.apply(current, availableExtensions);
             Set<String> extensionsRemoved = diff.apply(availableExtensions, current);
@@ -326,7 +388,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
         // not included in this test.
         if (ExtensionConf.CURRENT.isModified()) {
             op = Util.getReadAttributeOperation(null, "product-version");
-            result = DomainTestUtils.executeForResult(op, masterClient);
+            result = DomainTestUtils.executeForResult(op, primaryClient);
             if (!result.asString().startsWith(ExtensionConf.CURRENT.getName())) {
                 fail(String.format("The ExtensionConf.CURRENT has extensions added or removed but it no longer points to the current release. " +
                         "Modify this test adding new ExtensionConf enums for each previous releases undefined in this test by using the list of extensions added or removed on ExtensionConf.CURRENT." +
@@ -337,7 +399,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
         op = Util.getEmptyOperation(READ_CHILDREN_RESOURCES_OPERATION, null);
         op.get(CHILD_TYPE).set(HOST_EXCLUDE);
 
-        result = DomainTestUtils.executeForResult(op, masterClient);
+        result = DomainTestUtils.executeForResult(op, primaryClient);
 
         Set<String> processedExclusionsIds = new HashSet<>();
         for (Property prop : result.asPropertyList()) {
@@ -351,18 +413,19 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
 
             //check duplicated extensions
             Assert.assertTrue(String.format (
-                    "There are duplicated extensions declared for %s host-exclude", name),
+                            "There are duplicated extensions declared for %s host-exclude", name),
                     excludedExtensions.size() == new HashSet<>(excludedExtensions).size()
             );
 
             //check we have defined the current host-exclude configuration in the test
             ExtensionConf confPrevRelease = ExtensionConf.forName(name);
             Assert.assertNotNull(String.format(
-                    "This host-exclude name is not defined in this test: %s", name),
+                            "This host-exclude name is not defined in this test: %s", name),
                     confPrevRelease);
 
-            //check that available extensions - excluded extensions = expected extensions in a previous release.
-            Set<String> expectedExtensions = ExtensionConf.forName(name).getExtensions(isEeGalleonPack);
+            //check that available extensions - excluded extensions = expected extensions in a previous release - removed
+            Set<String> expectedExtensions = ExtensionConf.forName(name).getExtensions(isFullDistribution, isPreviewGalleonPack);
+            expectedExtensions.removeAll(ExtensionConf.forName(MAJOR).getRemovedExtensions());
 
             Set<String> extensionsUnderTest = new HashSet<>(availableExtensions);
             extensionsUnderTest.removeAll(excludedExtensions);
@@ -382,8 +445,10 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
 
         // Verifies all the exclusions Id added as configurations for this test are defined as host exclusions in the current server release
         for(ExtensionConf extensionConf : ExtensionConf.values()) {
-            if (extensionConf != ExtensionConf.CURRENT && !processedExclusionsIds.contains(extensionConf.getName())) {
-                if (!extensionConf.getExtensions(isEeGalleonPack).equals(availableExtensions)) {
+            if (extensionConf != ExtensionConf.CURRENT && extensionConf.isSupported() && !processedExclusionsIds.contains(extensionConf.getName())) {
+                Set<String> extensions = extensionConf.getExtensions(isFullDistribution, isPreviewGalleonPack);
+                extensions.removeAll(ExtensionConf.forName(MAJOR).getRemovedExtensions());
+                if (!extensions.equals(availableExtensions)) {
                     fail(String.format("The %s exclusion id is not defined as host exclusion for the current release.", extensionConf.getName()));
                 }
             }
@@ -391,7 +456,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
     }
 
     /**
-     * Retrieve the list of all modules which export locally a resource that implements a org.jboss.as.controller.Extension.
+     * Retrieve the list of all modules which export locally a resource that implements org.jboss.as.controller.Extension.
      * This list is considered the list of all available extensions that can be added to a server.
      *
      * It is assumed that the module which is added as an extension has the org.jboss.as.controller.Extension service as
@@ -419,7 +484,7 @@ public class HostExcludesTestCase extends BuildConfigurationTestBase {
     }
 
     private static File[] getModuleRoots() throws IOException {
-        Path layersRoot = Paths.get(masterConfig.getModulePath()) .resolve("system").resolve("layers");
+        Path layersRoot = Paths.get(primaryConfig.getModulePath()) .resolve("system").resolve("layers");
         DirectoryStream.Filter<Path> filter = entry -> {
             File f = entry.toFile();
             return f.isDirectory() && !f.isHidden();
