@@ -22,7 +22,7 @@
 
 package org.jboss.as.test.integration.ee.injection.resource.enventry.shared;
 
-import javax.ejb.Local;
+import jakarta.ejb.Local;
 
 /**
  * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
