@@ -21,10 +21,9 @@
 */
 package org.jboss.as.test.integration.jpa.dsrestart;
 
-import javax.ejb.Stateless;
+import jakarta.ejb.Stateless;
 
 /**
- *
  * @author <a href="kabir.khan@jboss.com">Kabir Khan</a>
  */
 @Stateless
