@@ -21,8 +21,8 @@
  */
 package org.jboss.as.test.integration.weld.builtin;
 
-import javax.ws.rs.ApplicationPath;
-import javax.ws.rs.core.Application;
+import jakarta.ws.rs.ApplicationPath;
+import jakarta.ws.rs.core.Application;
 
 @ApplicationPath("/cdibuiltin")
 public class CDIBuiltInApplication extends Application {
