@@ -43,6 +43,7 @@ import org.jboss.shrinkwrap.api.Archive;
 import org.jboss.shrinkwrap.api.ShrinkWrap;
 import org.jboss.shrinkwrap.api.spec.JavaArchive;
 import org.junit.Assert;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.wildfly.common.function.ExceptionSupplier;
@@ -57,6 +58,7 @@ import javax.naming.Context;
 
 @ServerSetup(TwoConnectorsEJBFailoverTestCase.ServerSetupTask.class)
 @RunWith(Arquillian.class)
+@Ignore("WFLY-17605")
 public class TwoConnectorsEJBFailoverTestCase extends AbstractClusteringTestCase {
 
     private static final int COUNT = 20;
@@ -88,9 +90,9 @@ public class TwoConnectorsEJBFailoverTestCase extends AbstractClusteringTestCase
     }
 
     /*
-     * Set up JNDI properties to support HTTP based EJB client invocations via HTTP Upgrade
+     * Set up JNDI properties to support HTTP based Jakarta Enterprise Beans client invocations via HTTP Upgrade
      *
-     * NOTE: there are several ways to connect to the EJB container on the server:
+     * NOTE: there are several ways to connect to the Jakarta Enterprise Beans container on the server:
      *   protocol           URL
      *   remoting           remote://localhost:4447
      *   HTTP Upgrade       remote+http://localhost:8080
@@ -228,7 +230,7 @@ public class TwoConnectorsEJBFailoverTestCase extends AbstractClusteringTestCase
         public ServerSetupTask() {
             this.builder.node(TWO_NODES)
                     .setup("/socket-binding-group=standard-sockets/socket-binding=remoting:add(port=4447)")
-                    .setup("/subsystem=remoting/connector=remoting-connector:add(socket-binding=remoting, security-realm=ApplicationRealm)")
+                    .setup("/subsystem=remoting/connector=remoting-connector:add(socket-binding=remoting, sasl-authentication-factory=application-sasl-authentication)")
                     .setup("/subsystem=remoting/connector=remoting-connector/property=SSL_ENABLED:add(value=false)")
                     // this step results in a capabilities error if the list is not formatted correctly for CLI
                     .setup("/subsystem=ejb3/service=remote:list-add(name=connectors, value=remoting-connector)")
