@@ -36,8 +36,8 @@ import java.util.Map;
 import java.util.ServiceLoader;
 import java.util.Set;
 
-import javax.servlet.ServletContainerInitializer;
-import javax.servlet.annotation.HandlesTypes;
+import jakarta.servlet.ServletContainerInitializer;
+import jakarta.servlet.annotation.HandlesTypes;
 
 import org.jboss.as.ee.structure.DeploymentType;
 import org.jboss.as.ee.structure.DeploymentTypeMarker;
@@ -223,10 +223,8 @@ public class ServletContainerInitializerDeploymentProcessor implements Deploymen
                     if (!servletContainerInitializerClassName.isEmpty()) {
                         // Instantiate the ServletContainerInitializer
                         ServletContainerInitializer service = (ServletContainerInitializer) classLoader.loadClass(servletContainerInitializerClassName).newInstance();
-                        if (service != null) {
-                            if(sciClasses.add(service.getClass())) {
-                                scis.add(service);
-                            }
+                        if (service != null && sciClasses.add(service.getClass())) {
+                            scis.add(service);
                         }
                     }
                     servletContainerInitializerClassName = reader.readLine();
